const config = require('config')
const { get } = require('lodash')

const unsubscribeSlug = config.get('unsubscribe.url')
const inviteReviewerPath = config.get('invite-reviewer.url')

const {
  Email,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')

const { getEmailCopy } = require('./emailCopy')

module.exports = {
  async sendInvitations({
    resend,
    baseUrl,
    fragment,
    UserModel,
    timestamp,
    collection,
    invitation,
    invitedUser,
  }) {
    const fragmentHelper = new Fragment({ fragment })
    const { title, abstract } = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const {
      activeAuthors: authors,
      submittingAuthor,
    } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const subjectBaseText = `${collection.customId}: Review`
    const titleText = `The manuscript titled "${title}" by ${
      submittingAuthor.firstName
    } ${submittingAuthor.firstName}`

    let queryParams = {
      invitationId: invitation.id,
      agree: true,
    }

    const detailsPath = `/projects/${collection.id}/versions/${
      fragment.id
    }/details`

    const declineLink = services.createUrl(baseUrl, inviteReviewerPath, {
      ...queryParams,
      agree: false,
      fragmentId: fragment.id,
      collectionId: collection.id,
      invitationToken: invitedUser.invitationToken,
    })

    let agreeLink = services.createUrl(baseUrl, detailsPath, queryParams)

    if (!invitedUser.isConfirmed) {
      queryParams = {
        ...queryParams,
        email: invitedUser.email,
        token: invitedUser.passwordResetToken,
        collectionId: collection.id,
        fragmentId: fragment.id,
        agree: true,
      }
      agreeLink = services.createUrl(baseUrl, inviteReviewerPath, queryParams)
    }

    const email = new Email({
      type: 'user',
      toUser: {
        email: invitedUser.email,
        name: `${invitedUser.firstName} ${invitedUser.lastName}`,
      },
      content: {
        authors,
        abstract,
        agreeLink,
        declineLink,
        subject: `${subjectBaseText} Requested`,
        detailsLink: services.createUrl(baseUrl, detailsPath),
        signatureName: get(collection, 'handlingEditor.name', 'Hindawi'),
        unsubscribeLink: services.createUrl(baseUrl, unsubscribeSlug, {
          id: invitedUser.id,
        }),
      },
    })

    sendInvitedUserEmail({
      email,
      titleText,
      resend,
      timestamp,
    })
  },
}

const sendInvitedUserEmail = async ({
  email,
  titleText,
  resend,
  timestamp,
}) => {
  const emailType =
    resend === true ? 'reviewer-resend-invitation ' : 'reviewer-invitation'
  const daysExpected = resend === true ? 0 : 14

  const { html, text } = email.getBody({
    isReviewerInvitation: true,
    body: getEmailCopy({
      emailType,
      titleText,
      expectedDate: services.getExpectedDate({ timestamp, daysExpected }),
    }),
  })

  email.sendEmail({ html, text })
}
